

__doc__ = """ZMailIn product module."""

# Version string. Usually updated automatically by CVS. 

# $Id: ZMailIn.py,v 1.9 2001/03/09 10:37:28 andyd Exp $

__version__='$Revision: 1.9 $'[11:-2]  

# Imports go here.  Fosters, Carling etc. 
import string, rfc822, multifile, StringIO, mimetools, re

from Globals import HTMLFile, default__class_init__
from Globals import MessageDialog

from OFS.SimpleItem import SimpleItem

manage_addZMailInForm = HTMLFile('dtml/ZMailInAdd', globals()) 

def manage_addZMailIn(self, id, title='', email_address='', method='', REQUEST=None):
    """Add a ZMailIn to a folder."""
    self._setObject(id, ZMailIn(id, self, title, email_address, method, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

localpart_re=re.compile('<?(.*)@.*>?')

class ZMailIn(SimpleItem): 
    """ZMailIn object. """

    meta_type = 'ZMailIn Client'

    index_html = HTMLFile('dtml/index', globals()) # View Interface

    manage_main = HTMLFile('dtml/ZMailInEdit', globals()) # Management Interface

    manage_options = ( # what management options are there?
        {'label': 'Edit',       'action': 'manage_main'},
        {'label': 'Security',   'action': 'manage_access'},
    ) 

    __ac_permissions__=( # what permissions make sense for us? 
        ('View management screens', ['manage_tabs','manage_main'],('Manager',)  ),
        ('Change permissions',      ['manage_access'],('Manager',)              ),
        ('Change ZMailIn'     ,     ['manage_edit'],('Manager',)                ),
        ('Post mail to ZMailIn',    ['postZMailMessage'],('Anonymous','Manager')),
    )

    def __init__(self, id, parent, title='', email_address='', method='',REQUEST=None): 
        """initialise a new instance of ZMailIn"""

        wrapped_self = self.__of__(parent)
        self.id = id
        self.title = title
        self.email_address = email_address
        self.method = method
        
        # Catalog this object
        catalog = wrapped_self.restrictedTraverse("/GlobalZMailInCatalog",None)
        catalog.catalog_object(wrapped_self, string.join(wrapped_self.getPhysicalPath(),'/'))

    def manage_edit(self, title, email_address, method, REQUEST=None): 
        """ Edit """
        self.title = title
        self.email_address = email_address
        self.method = method

        # Catalog this object
        catalog = self.restrictedTraverse("/GlobalZMailInCatalog",None)
        catalog.catalog_object(self, string.join(self.getPhysicalPath(),'/'))

        if REQUEST is not None:
            return MessageDialog(
                title = 'Edited',
                message = "Properties for <I>%s</I> changed." % self.id,
                action = './manage_main',
        )
        
    def manage_beforeDelete(self, container, unknown):
        # Remove the object from the catalog
        catalog = self.restrictedTraverse("/GlobalZMailInCatalog",None)
        catalog.uncatalog_object(string.join(self.getPhysicalPath(),'/'))
        # cha-ching

    def postZMailMessage(self, file=None):
        """
        This method will create a dictionary containing all the information about the Mail Message
        The dictionary is then handed to the method stated in self.method for processing
        """

        if not file:
            raise IOError, 'No Mail Message Supplied'
        
        sio = StringIO.StringIO(file)  # Put it in a StringIO so we can play with it over and over
        m = rfc822.Message(sio)  # Render the message through rfc822 so we can get the headers
        sio.seek(0)

        theMail = {}  # A New Dictionary
        
        # Place all the email header in the headers dictionary in theMail
        headers = theMail['headers'] = {}
        for (name,value) in m.items():
            headers[name]=value
        
        # put localpart in theMail
        address_to = headers.get('envelope-to',headers.get('to',''))
        if not address_to:
            raise KeyError,'neither envelope-to or to found in supplied mail message'

        theMail['localpart'] = self._getLocalPart(address_to)
         
        # Process mime encoded mail messages
        mf = multifile.MultiFile(sio,0)
        theMail['attachments'] = {}
        self._unpack_part(theMail,mf,1)

        if self.method:
            return self.restrictedTraverse(self.method)(theMail=theMail)
        
        self.REQUEST.RESPONSE.notFoundError('ZMailIn method not specified')

    def _unpack_part(self,theMail, file, partno):
        # Will unpack a multipart message in the MIME format and enter it in to
        # the dictionary theMail

        headers = mimetools.Message(file)
        type = headers.getmaintype()
        if type == 'multipart':
            boundary = headers.getparam("boundary")
            file.push(boundary)
            file.readlines()
            while not file.last:
                file.next()
                partno = partno + 1
                self._unpack_part(theMail, file, partno)
            file.pop()
            return

        name = headers.getparam("name")
        subtype = headers.getsubtype()
        if type=='text' and subtype=='plain' and not name:
            # overwrite the body so that we get _only_ the body text.
            # (Uses the plain version of the body in all situations.
            #  HTML formated body will be in the attachments dictionary)
            theMail['body'] = file.read()
        else:
            if not name:
                name = 'part%d' % (partno,)
                name = name + '-'+subtype

            encoding = headers.getencoding()


            outfile = StringIO.StringIO()
            if encoding == '7bit':
                mimetools.copyliteral(file,outfile)
            else:
                mimetools.decode(file,outfile,encoding)

            theMail['attachments'][name]=outfile.getvalue()
            outfile.close()

    def _getLocalPart(self, address):
        str1 = address[:string.find(address,'@')]
        if string.find(str1,'<')>-1:
            return str1[string.find(str1,'<')+1:]
        return str1            
    
default__class_init__(ZMailIn)
