__doc__ = """ZMailMessage product module."""

# $Id: ZMailMessage.py,v 1.6 2001/03/06 10:33:03 andyd Exp $

# Version string.

__version__='$Revision: 1.6 $'[11:-2] 

# Imports go here
import rfc822, string, urllib

from Products.ZCatalog.CatalogAwareness import CatalogAware   # Makes messages catalog aware
from Products.ZCatalog import ZCatalog
from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

from OFS.Folder import Folder
from Acquisition import Implicit
import AccessControl.Role

manage_addZMailMessageForm = HTMLFile('dtml/ZMailMessageAdd', globals()) 

def manage_addZMailMessage(self, id, title='', subject='', date='', to='', sender='', replyto='', body='', headers='', otherInfo='', REQUEST=None):
    """Add a ZMailMessage to a folder."""
    self._setObject(id, ZMailMessage(id, self, title, subject, date, to, sender, replyto, body, headers, otherInfo))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class ZMailMessage(Folder): 
    """ZMailMessage object."""
        
    meta_type = 'ZMailMessage'
    
    manage_options = ( # the management options available
                       {'label': 'Contents',    'action': 'manage_main'},
                       {'label': 'View',        'action': ''}, # defaults to index_html
                       {'label': 'Properties',  'action': 'manage_propertiesForm'},
                       {'label': 'Security',    'action': 'manage_access'},
                       ) 

    __ac_permissions__=( # what permissions make sense for us? 
                         ('View management screens', ('manage_tabs','manage_main')),
                         ('Change permissions',      ('manage_access',)           ),
                         ('Change ZMailMessages',    ('manage_edit',)             ),
                         ('View ZMailMessage',       ('',)                        ),
                         )

    index_html = HTMLFile('dtml/message_index', globals()) # View Interface
    
    manage_propertiesForm = HTMLFile('dtml/ZMailMessageEdit', globals()) # Properties Interface

    def __init__(self, id, parent, title='', subject='', date='', to='', sender='', replyto='', body='', headers='', otherInfo=''): 
        """initialise a new instance of ZMailMessage"""

        wrapped_self = self.__of__(parent)

        self.id = id
        self.title = title
        self.subject = subject
        self.date = date
        self.sortableDate = self._convertToSortableDate(date)
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers
        self.otherInfo = otherInfo
        
        # Catalog this object
        catalog = wrapped_self.restrictedTraverse("ZMailInCatalog",None)
        if catalog:
            catalog.catalog_object(wrapped_self, string.join(wrapped_self.getPhysicalPath(),'/'))

    def _convertToSortableDate(self, date):
        # Converts a date in to a string which can be sorted to produce the correct results
        if not date:
            return ""
        dateTuple = rfc822.parsedate(date)
        return str('%04d%02d%02d%02d%02d%02d' % (dateTuple[0],dateTuple[1],dateTuple[2],dateTuple[3],dateTuple[4],dateTuple[5]))

    def highlightKeyword(self, keyword):
        # Given a keyword this method will put HTML around it to make it highlighted
        keyword2 = urllib.unquote_plus(keyword)
        return string.replace(self.body,keyword2,'<FONT COLOR="#008000"><B>'+keyword2+'</B></FONT>')

    def manage_edit(self, title, subject, date, to, sender, replyto, body, headers, REQUEST=None): 
        """After editing the Object this function is called."""
        self.title = title
        self.subject = subject
        self.date = date
        self.sortableDate = self._convertToSortableDate(date)
        self.to = to
        self.sender = sender
        self.replyto = replyto
        self.body = body
        self.headers = headers

        # Catalog this object
        catalog = self.restrictedTraverse("ZMailInCatalog",None)
        catalog.catalog_object(self, string.join(self.getPhysicalPath(),'/'))
        
        if REQUEST is not None:
            return MessageDialog(
                                 title = 'Edited',
                                 message = "Properties for <I>%s</I> changed." % self.id,
                                 action = './manage_main',
                                 )

    def manage_beforeDelete(self, container, unknown):
        # Remove the object from the catalog
        catalog = self.restrictedTraverse("ZMailInCatalog",None)
        catalog.uncatalog_object(string.join(self.getPhysicalPath(),'/'))
        # cha-ching
